class RnnRbm:
    '''Simple class to train an RNN-RBM from MIDI files and to generate sample
        sequences.'''

            def __init__(self, n_hidden=150, n_hidden_recurrent=100, lr=0.001,
                    r=(21, 109), dt=0.3):
                '''Constructs and compiles Theano functions for training and sequence
                        generation.

                        n_hidden : integer
                          Number of hidden units of the conditional RBMs.
                          n_hidden_recurrent : integer
                            Number of hidden units of the RNN.
                            lr : float
                              Learning rate
                              r : (integer, integer) tuple
                                Specifies the pitch range of the piano-roll in MIDI note numbers, including
                                  r[0] but not r[1], such that r[1]-r[0] is the number of visible units of the
                                    RBM at a given time step. The default (21, 109) corresponds to the full range
                                      of piano (88 notes).
                                      dt : float
                                        Sampling period when converting the MIDI files into piano-rolls, or
                                          equivalently the time difference between consecutive time steps.'''

                                                  self.r = r
                                                          self.dt = dt
                                                                  (v, v_sample, cost, monitor, params, updates_train, v_t,
                                                                          updates_generate) = build_rnnrbm(r[1] - r[0], n_hidden,
                                                                                  n_hidden_recurrent)

                                                                          gradient = T.grad(cost, params, consider_constant=[v_sample])
                                                                                                   updates_train.update(((p, p - lr * g) for p, g in zip(params,
                                                                                                       gradient)))
                                                                                                   self.train_function = theano.function([v], monitor,
                                                                                                           updates=updates_train)
                                                                                                   self.generate_function = theano.function([], v_t,
                                                                                                           updates=updates_generate)

                                                                                                   def train(self, files, batch_size=100, num_epochs=200):
                                                                                                       '''Train the RNN-RBM via stochastic gradient descent (SGD) using MIDI
                                                                                                                                   files converted to piano-rolls.

                                                                                                                                   files : list of strings
                                                                                                                                     List of MIDI files that will be loaded as piano-rolls for training.
                                                                                                                                     batch_size : integer
                                                                                                                                       Training sequences will be split into subsequences of at most this size
                                                                                                                                         before applying the SGD updates.
                                                                                                                                         num_epochs : integer
                                                                                                                                           Number of epochs (pass over the training set) performed. The user can
                                                                                                                                             safely interrupt training with Ctrl+C at any time.'''

                                                                                                                                                     assert len(files) > 0, 'Training set is empty!' \
                                                                                                                                                             ' (did you download the data files?)'
                                                                                                                                                                                                    dataset = [midiread(f, self.r,
                                                                                                                                                                                                        self.dt).piano_roll.astype(theano.config.floatX)
                                                                                                                                                                                                        for f in files]

                                                                                                                                                                                                    try:
                                                                                                                                                                                                        for epoch in xrange(num_epochs):
                                                                                                                                                                                                            numpy.random.shuffle(dataset)
                                                                                                                                                                                                                                                                costs = []

                                                                                                                                                                                                                                                                                for s, sequence in enumerate(dataset):
                                                                                                                                                                                                                                                                                    for i in xrange(0, len(sequence), batch_size):
                                                                                                                                                                                                                                                                                        cost = self.train_function(sequence[i:i + batch_size])
                                                                                                                                                                                                                                                                                                                                                            costs.append(cost)

                                                                                                                                                                                                                                                                                                                                                                            print 'Epoch %i/%i' % (epoch + 1, num_epochs),
                                                                                                                                                                                                                                                                                                                                                                                            print numpy.mean(costs)
                                                                                                                                                                                                                                                                                                                                                                                                            sys.stdout.flush()

                                                                                                                                                                                                    except KeyboardInterrupt:
                                                                                                                                                                                                        print 'Interrupted by user.'

                                                                                                                                                                                                                                                                                                                                                                                                                                        def generate(self, filename, show=True):
                                                                                                                                                                                                                                                                                                                                                                                                                                            '''Generate a sample sequence, plot the resulting piano-roll
                                                                                                                                                                                                                                                                                                                                                                                                                                                    and save
                                                                                                                                                                                                                                                                                                                                                                                                                                                    it as a MIDI file.

                                                                                                                                                                                                                                                                                                                                                                                                                                                    filename : string
                                                                                                                                                                                                                                                                                                                                                                                                                                                      A MIDI file will be created at this location.
                                                                                                                                                                                                                                                                                                                                                                                                                                                      show : boolean
                                                                                                                                                                                                                                                                                                                                                                                                                                                        If True, a piano-roll of the generated sequence will be
                                                                                                                                                                                                                                                                                                                                                                                                                                                        shown.'''

                                                                                                                                                                                                                                                                                                                                                                                                                                                                piano_roll = self.generate_function()
                                                                                                                                                                                                                                                                                                                                                                                                                                                                        midiwrite(filename, piano_roll, self.r,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                self.dt)
                                                                                                                                                                                                                                                                                                                                                                                                                                                                        if show:
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            extent = (0, self.dt
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    * len(piano_roll)) +
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * self.r
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            pylab.figure()
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        pylab.imshow(piano_roll.T,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                origin='lower',
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                aspect='auto',
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                interpolation='nearest',
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                cmap=pylab.cm.gray_r,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                extent=extent)
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        pylab.xlabel('time
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    (s)')
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                pylab.ylabel('MIDI
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                note
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                number')
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            pylab.title('generated
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            piano-roll')


                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            if
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                __name__
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                ==
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                '__main__':
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    model
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    =
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    RnnRbm()
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        model.train(glob.glob('../data/Nottingham/train/*.mid'))
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            model.generate('sample1.mid')
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                model.generate('sample2.mid')
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    pylab.show()
